/**
 * @file mainwindow.h
 * @brief client
 * @author Bagrova Polina
 */
#include "MainWindow.h"
#include "ui_mainwindow.h"
#include <QMessageBox>
#include "camera/CCameraManager.h"
#include "device/CDeviceManager.h"
#include "file/CFileManager.h"
#include "./ui/setting/DlgMainSetting.h"
#include "ui/setting/FormLogin.h"
#include "./socket/CWebSocketClient.h"
#include <assert.h>
#include <QMetaType>
#include "ui/setting/DlgSetting.h"
#include "./gps/CGPSManager.h"
#include "audio/CMcAdptManager.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent),
      ui(new Ui::MainWindow),
      m_actCamGroup(this),
      m_pVideoMng(nullptr),
      m_pDevMng(nullptr),
      m_pWebSock(nullptr)
{
    ui->setupUi(this);
    qRegisterMetaType<QPixmap>("QPixmap&"); //放在构造函数中使用

    m_pWinShow[0] = ui->cam1;
    m_pWinShow[1] = ui->cam2;
    m_pWinShow[2] = ui->cam3;
    m_pWinShow[3] = ui->cam4;

    onGpsMasked(true);
    onSpeedAlarm(false);
    onNetworkAlarm(true);

    // 控件初始化
    m_actCamGroup.addAction(ui->actionactShowOneCam);
    m_actCamGroup.addAction(ui->actionactShowFourCam);
    ui->actionactShowFourCam->setChecked(true);

    // 视频管理模块
    m_pVideoMng = dynamic_cast<CCameraManager *>(QueryPlugin("CCameraManager"));
    if (m_pVideoMng)
    {
        for (int i = 0; i < MAX_CAMERA_COUNT; i++)
        {
            setCamViewStyle(m_pWinShow[i]);
            //            m_pWinShow[i]->setScaledContents(true);
            CCameraDevice *pCam = m_pVideoMng->getCameraByIndex(i);
            if (pCam)
            {
                connect(pCam, SIGNAL(notifyUpdateImage(int, QPixmap &)), this, SLOT(onUpdateImage(int, QPixmap &)));
            }
        }

        m_pVideoMng->setVideoWId(0, ui->cam1->winId());
        m_pVideoMng->setVideoWId(1, ui->cam2->winId());
        m_pVideoMng->setVideoWId(2, ui->cam3->winId());
        m_pVideoMng->setVideoWId(3, ui->cam4->winId());
    }

    //执行多次，每半分钟查询一次在线设备
    m_pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    if (m_pDevMng)
    {
        connect(m_pDevMng, SIGNAL(notifyShowLogin()), this, SLOT(onShowLogin()));
        connect(m_pDevMng, SIGNAL(notifyCloseApp()), this, SLOT(closeApp()));
        connect(m_pDevMng, SIGNAL(notifyNetworkAlarm(bool)), this, SLOT(onNetworkAlarm(bool)));
    }

    // GPS信号 速度相关的告警状态指示
    m_pGpsMng = dynamic_cast<CGPSManager *>(QueryPlugin("CGPSManager"));
    if (m_pGpsMng)
    {
        connect(m_pGpsMng, SIGNAL(notifyGpsMasked(bool)), this, SLOT(onGpsMasked(bool)));
        connect(m_pGpsMng, SIGNAL(notifySpeedAlarm(bool)), this, SLOT(onSpeedAlarm(bool)));
    }

    // 手咪状态指示
    m_pAudioMng = dynamic_cast<CMcAdptManager *>(QueryPlugin("CMcAdptManager"));
    if (m_pAudioMng)
    {
        connect(m_pAudioMng, SIGNAL(notifyAudioStatus(int)), this, SLOT(onAudioStatus(int)));
    }
	m_pFileMng = dynamic_cast<CFileManager *>(QueryPlugin("CFileManager"));
	if (m_pFileMng)
	{
		connect(m_pFileMng, SIGNAL(notifyCopyFile(int, int)), this, SLOT(onCopyFile(int, int)));
	}
}

// 关闭程序
void MainWindow::closeApp()
{
    close();
}

void MainWindow::setCamViewStyle(QWidget *pWidget)
{
    //    //①设置界面基本颜色
    pWidget->setAutoFillBackground(true); //属性控制小部件背景是否自动填充
    //这样设置就可以了，但是我的需求是窗体要有边框，故需要添加下边一步

    //②使用样式变添加边框，设置透明背景
    pWidget->setStyleSheet(QString("background-color:rgba(255,255,255,30);"
                                   "color:white"));
}

void MainWindow::updateRuningStatus()
{
    if (ui->btn_stop->isChecked() || ui->btn_audio->isChecked() || ui->btn_mask->isChecked() || ui->btn_newrok->isChecked())
    {
        ui->btn_status->setChecked(false);
    }
    else
    {
        ui->btn_status->setChecked(true);
    }
}

// gps遮挡消息处理
void MainWindow::onGpsMasked(bool bMasked)
{
    ui->btn_mask->setChecked(bMasked);
    updateRuningStatus();
}

// 车速异常消息处理
void MainWindow::onSpeedAlarm(bool bAlarmd)
{
    ui->btn_stop->setChecked(bAlarmd);
    updateRuningStatus();
}

// 手咪异常消息处理
void MainWindow::onAudioStatus(int status)
{
    bool bOK = status == 0 ? false : true;
    ui->btn_audio->setChecked(bOK);
    updateRuningStatus();
}
// u盘文件复制
void MainWindow::onCopyFile(int files, int processed)
{
	
	if (0 == processed)
	{
		progDlg = new QProgressDialog();
	    progDlg->setWindowTitle("Please wait backuping..."); 
	    progDlg->setFixedWidth(300);
	    progDlg->setRange(0, 100);
	    progDlg->show();
		progDlg->exec();
		return;
	}
	int percentage = (processed*100)/files;
	if (processed < files)
	{
		
		 progDlg->setValue(percentage);
		 // if(progDlg->wasCanceled())
        //	progDlg->setHidden(true);//隐藏对话框
	}
	else if (processed == files)
	{
		 if(percentage != 100)
      		percentage = 100;
		 progDlg->setValue(percentage);
		  QTimer::singleShot(5000, this, SLOT(onCopyTimeout()));
	}
		
}
void MainWindow::onCopyTimeout()
{
	progDlg->hide();
	progDlg->done(1);
	delete progDlg;
	progDlg = NULL;
	
}

// 网络异常处理
void MainWindow::onNetworkAlarm(bool bAlarmd)
{
    ui->btn_newrok->setChecked(bAlarmd);
    updateRuningStatus();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::onUpdateImage(int idx, QPixmap &pixmap)
{
    if (idx < MAX_CAMERA_COUNT && idx >= 0)
    {
        m_pWinShow[idx]->setPixmap(pixmap);
    }
}

// 心跳命令
bool MainWindow::ping()
{
    assert(m_pDevMng);
    m_pDevMng->ping();
}

// 发送状态信息
bool MainWindow::sendCommonStatus()
{
    assert(m_pDevMng);
    m_pDevMng->sendCommonStatus();
}

// 上报GPS 速度信息
bool MainWindow::sendGpsSpeed()
{
    assert(m_pDevMng);
    m_pDevMng->sendGpsSpeed();
}

// 主界面初始化
int MainWindow::Initialize()
{

    m_pDevMng->connect2Slots();

    return 1;
}

// 显示登陆界面
void MainWindow::onShowLogin()
{
    FormLogin dlg(this);
    dlg.exec();
}

void MainWindow::keyPressEvent(QKeyEvent *event)
{
    if (event->modifiers() & Qt::ControlModifier)
    {
        if (event->key() == Qt::Key_F12) //单键触发
        {
            onSettingClicked();
        }
        else if (event->key() == Qt::Key_F4) //单键触发
        {
            this->closeApp();
        }
    }
}
void MainWindow::onSettingClicked()
{
    DlgSetting dlg(this);
    connect(&dlg, SIGNAL(notifyClose()), this, SLOT(closeApp()));
    dlg.exec();
    disconnect(&dlg, SIGNAL(notifyClose()), 0, 0);
}

// 点击设置按钮
void MainWindow::on_actSetting_triggered()
{
    CConfigManager *pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (pConfigMng)
    {
        DlgMainSetting dlg(*pConfigMng);

        dlg.setModal(true);
        dlg.showFullScreen();
        // dlg.exec();
    }
}

// 点击显示单一画面
void MainWindow::on_actionactShowOneCam_triggered()
{
}
