#include "CCameraManager.h"
#include "../socket/TcpServer.h"
#include "../base_module/factory.h"
#include "config/CConfigManager.h"
#include "timer/CTimerManager.h"
#include "device/CDeviceManager.h"
#include "socket/CWebSocketClient.h"
#include "file/CFileManager.h"
#include "../log/easyloger.h"
#include <QMutexLocker>
#include <assert.h>
#include "jt808/jt808_common.h"
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////              CCameraManager              /////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

REGISTER_MODULE(CCameraManager, "CCameraManager")
LOG_DEFINITION(CCameraManager)

const QString CAMERA_CONFIG_FILE = "./config/cam_cfg.xml";
const QString VIDEO_DIR_NAME = "video/";
const QString IMAGE_DIR_NAME = "image/";

CCameraManager::CCameraManager()
    : QObject(),
      IModule(),
      m_pTcpModule(nullptr),
      m_pTimerMng(nullptr),
      m_pConfigMng(nullptr),
      m_pDevMng(nullptr),
      m_pWebSock(nullptr),
      m_pFileMng(nullptr),
      m_pCamDb(nullptr)
{
    memset(&m_vctCamWID, 0, sizeof(m_vctCamWID));
    LOG_CREATE(CCameraManager, "camera");
}

CCameraManager::~CCameraManager()
{
    foreach (CCamConfig *pInfo, m_mapCamConfig)
    {
        SAFE_DELETE(pInfo);
    }

    std::cout << name() << std::endl;
}

//初始化操作
bool CCameraManager::Initialize()
{
	
    InitializeDB();
    m_pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    assert(m_pDevMng);

    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        m_vctCam[i].m_pCamMng = this;
    }

    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

// 创建数据库
bool CCameraManager::InitializeDB()
{
    // 注册数据库
    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (m_pConfigMng)
    {
        m_pCamDb = m_pConfigMng->getDatabase(MODULE_CAMERA_MNG, CAMERA_DB_NAME);
        if (!m_pCamDb)
        {
            m_pCamDb = m_pConfigMng->regDatabase(MODULE_CAMERA_MNG, CAMERA_DB_NAME);
            m_pCamDb->regKey("video_save_interval", 600);
            m_pCamDb->save();
        }
    }

    return true;
}

// 保存数据库
bool CCameraManager::saveConfig()
{
    if (!m_pCamDb)
    {
        m_pCamDb->save();
        return true;
    }
    return false;
}

void CCameraManager::shutdown()
{
    m_webCamFinder.shutdown();
    IModule::shutdown();
}

// 恢复出厂配置回调函数
bool CCameraManager::restoreFactoryCallback(void *This, int type)
{
    CCameraManager *pThis = (CCameraManager *)This;
    return pThis->restoreFactory(type);
}

// 恢复出厂值
bool CCameraManager::restoreFactory(int type)
{
    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        if (!m_vctCam[i].isCameraIdel())
        {
            m_vctCam[i].stopRun();
        }
        /* code */
    }
}

// 获取摄像头
CCameraDevice *CCameraManager::getCameraByIndex(int index)
{
    if (index >= 0 && index < MAX_CAMERA_COUNT)
    {
        return &(m_vctCam[index]);
    }
    return nullptr;
}

// 视频保存间隔
int CCameraManager::getVideoInterval()
{
    int interval = 600;
    if (m_pCamDb)
    {
        interval = m_pCamDb->getValue("video_save_interval").toInt();
    }
    return interval;
}

// 设置视频保存间隔
bool CCameraManager::setVideoInterval(int interval)
{
    if (interval < 60 || interval > 3600 * 4)
    {
        return false;
    }

    if (m_pCamDb)
    {
        m_pCamDb->setValue("video_save_interval", interval);
    }
}

// 设置相机配置
void CCameraManager::setCamConfig(CCamConfig &cam_cfg)
{
    m_mtxCamCfg.lock();
    if (m_mapCamConfig.find(cam_cfg.m_strCamDev) == m_mapCamConfig.end())
    {
        CCamConfig *pCam_cfg = new CCamConfig(cam_cfg);
        m_mapCamConfig[cam_cfg.m_strCamDev] = pCam_cfg;
    }
    else
    {
        *(m_mapCamConfig[cam_cfg.m_strCamDev]) = cam_cfg;
    }
    m_mtxCamCfg.unlock();
}

void CCameraManager::setVideoWId(int index, WId win_id)
{
    if (index >= 0 && index < MAX_CAMERA_COUNT)
    {
        m_vctCamWID[index] = win_id;
        m_vctCam[index].setWinIndex(index);
        m_vctCam[index].setWindowID(win_id);
    }
}

bool CCameraManager::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_CAMERA_MNG);
    }
    return true;
}

QString CCameraManager::getVideoPath()
{
    if (m_pFileMng)
    {
        return m_pFileMng->getDataPath() + "video/";
    }
    else
    {
        return "./video/";
    }
}

QString CCameraManager::getSnapshotPath()
{
    if (m_pFileMng)
    {
        return m_pFileMng->getDataPath() + "image/";
    }
    else
    {
        return "./image/";
    }
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CCameraManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskVideoMain");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();

    //起主消息处理线程
    STARTUP_TASK(CameraTask);

    connect2Slots();

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            case CMD_VIDEO_SET_CAM_CFG:
                procSetCamConfig(pMsg);
                break;
            case TIMER_FIND_CAM_DEVICE:
                STARTUP_TASK(tskCameraFind);
                break;
            case TIMER_SAVE_TO_NEW_VIDEO:
                procSave2NewFile(pMsg);
                break;
            case CMD_START_GB28181_VIDEO:
                procStartGB28181Video(pMsg);
                break;
            case CMD_STOP_GB28181_VIDEO:
                procStopGB28181Video(pMsg);
                break;
            case CMD_START_JT808_RT_VIDEO:
                procStartJt808RTVideo(pMsg);
                break;
            case CMD_JT808_RT_VIDEO_CONTROL:
                procJt808RTVideoControl(pMsg);
                break;
            case CMD_START_JT808_HIS_VIDEO:
                procStartJt808HisVideo(pMsg);
                break;
            case CMD_JT808_HIS_VIDEO_CONTROL:
                procJt808HisVideoControl(pMsg);
                break;
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

// 信号槽简易信号处理
void CCameraManager::connect2Slots()
{

    m_pWebSock = dynamic_cast<CWebSocketClient *>(QueryPlugin("CWebSocketClient"));
    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    m_pFileMng = dynamic_cast<CFileManager *>(QueryPlugin("CFileManager"));

    assert(m_pConfigMng);
    m_pConfigMng->regRestoreCallback(restoreFactoryCallback, (void *)this);

    m_pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (m_pTimerMng)
    {
        int cam_finder_interval = m_pCamDb->getValue("cam_finder_interval").toInt();
        int video_save_interval = m_pCamDb->getValue("video_save_interval").toInt();

        // 这里做个限制,最小查找时间60秒间隔,防止查找过于频繁
        if (cam_finder_interval < 60)
        {
            cam_finder_interval = 60;
        }

        m_pTimerMng->RegTimer(MODULE_CAMERA_MNG, TIMER_FIND_CAM_DEVICE, cam_finder_interval);
        m_pTimerMng->RegTimer(MODULE_CAMERA_MNG, TIMER_SAVE_TO_NEW_VIDEO, video_save_interval, TIMER_TYPE_INTEGRAL);
    }

    STARTUP_TASK(tskCameraFind);

    m_pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    if (m_pFileMng)
    {
        makePath(m_pFileMng->getDataPath() + VIDEO_DIR_NAME);
        makePath(m_pFileMng->getDataPath() + IMAGE_DIR_NAME);
    }
}

// 开始gb28181推流
void CCameraManager::procStartGB28181Video(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    int idx = 0;
    int cam_idx = 0;
    int stream_code = 0;
    int rtp_port = 0;
    string rtp_ip = "";
    string ssrc = "";
    // 通信类型 int
    memcpy(&cam_idx, pMsg->szBuff + idx, sizeof(int));
    idx += sizeof(int);

    // 码流 int
    memcpy(&stream_code, pMsg->szBuff + idx, sizeof(int));
    idx += sizeof(int);

    // 端口 int
    memcpy(&rtp_port, pMsg->szBuff + idx, sizeof(int));
    idx += sizeof(int);

    // 对端IP 20
    rtp_ip = (char *)(pMsg->szBuff + idx);
    idx += 20;

    // SSRC
    ssrc = (char *)(pMsg->szBuff + idx);
    idx += 64;

    // 判断输入的INDEX 是否合法
    if (cam_idx < 0 || cam_idx >= MAX_CAMERA_COUNT)
    {
        cam_idx = 0;
    }

    m_vctCam[cam_idx].startGB28181Video(rtp_ip, rtp_port, ssrc, stream_code);
}

// 停止gb28181推流
void CCameraManager::procStopGB28181Video(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    int idx = 0;
    int channel = 0;

    // 通道号
    memcpy(&channel, pMsg->szBuff + idx, sizeof(int));
    idx += sizeof(int);

    if (channel >= 0 && channel < MAX_CAMERA_COUNT)
    {
        m_vctCam[channel].stopGB28181Video();
    }
}

// 开始JT808实时推流
void CCameraManager::procStartJt808RTVideo(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    string phone_num = "";

    int idx = 0;
    int channel = 0;
    int stream_code = 0;
    QString strIp;
    int tcp_port;
    int udp_port;
    int cam_idx = 0;

    if (m_pDevMng)
    {
        phone_num = m_pDevMng->getPhoneNumber().toString().toStdString();
    }

    // ip地址长度
    int ip_len = pMsg->szBuff[idx++];
    // ip地址
    strIp = QString((char *)&pMsg->szBuff[idx]).mid(0, ip_len);
    idx += ip_len;
    // tcp 端口
    tcp_port = two_char_to_short(&pMsg->szBuff[idx]);
    idx += 2;
    // udp 端口
    udp_port = two_char_to_short(&pMsg->szBuff[idx]);
    idx += 2;
    // 通道号
    channel = pMsg->szBuff[idx++];
    // 数据类型
    int date_type = pMsg->szBuff[idx++];
    // 码流类型
    stream_code = pMsg->szBuff[idx++];

    cam_idx = channel - 1;

    // 判断输入的INDEX 是否合法
    if (cam_idx < 0 || cam_idx >= MAX_CAMERA_COUNT)
    {
        cam_idx = 0;
    }

    m_vctCam[cam_idx].startJt808RTVideo(phone_num, strIp.toStdString(), tcp_port, udp_port, stream_code);
}

// jt808实时推流控制
void CCameraManager::procJt808RTVideoControl(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    int idx = 0;
    BYTE channel = pMsg->szBuff[idx++];
    BYTE ctrl_code = pMsg->szBuff[idx++];
    BYTE stop_code = pMsg->szBuff[idx++];
    BYTE stream_code = pMsg->szBuff[idx++];

    int cam_idx = channel - 1;

    // 判断输入的INDEX 是否合法
    if (cam_idx < 0 || cam_idx >= MAX_CAMERA_COUNT)
    {
        cam_idx = 0;
    }

    switch (ctrl_code)
    {
    case 0: //停止推流
    {
        // 0和2状态表示关闭视频
        if (stop_code == 0 || stop_code == 2)
        {
            m_vctCam[cam_idx].stopJt808RTVideo();
        }
        break;
    }
    case 1: // 换码流
    {
        m_vctCam[cam_idx].changeJt808RTStreamCode(stream_code);
        break;
    }
    case 2: // 暂停推流
    {
        m_vctCam[cam_idx].pauseJt808RTVideo();
        break;
    }
    case 3: // 继续推流
    {
        m_vctCam[cam_idx].continueJt808RTVideo();
        break;
    }
    case 4:
    default:
        break;
    }
}

// 开始JT808历史推流
void CCameraManager::procStartJt808HisVideo(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    string phone_num = "";
    // 目标IP
    // 目标tcp端口
    // 目标udp端口
    // 文件长度
    const int MAX_FILE_PATH_LEN = 260;
    int idx = 0;
    int channel = 0;
    int stream_code = 0;
    char szIP[20];
    char file_path[MAX_FILE_PATH_LEN];
    int tcp_port;
    int udp_port;
    int cam_idx = 0;
    int speed = 0;

    if (m_pDevMng)
    {
        phone_num = m_pDevMng->getPhoneNumber().toString().toStdString();
    }

    // ip地址这里取总长度20
    strncpy(szIP, (char *)&pMsg->szBuff[idx], 20);
    idx += 20;
    // tcp 端口
    tcp_port = two_char_to_short(&pMsg->szBuff[idx]);
    idx += 2;
    // udp 端口
    udp_port = two_char_to_short(&pMsg->szBuff[idx]);
    idx += 2;
    // 通道号
    channel = pMsg->szBuff[idx++];
    // 文件名,长度取
    strncpy(file_path, (char *)&pMsg->szBuff[idx], MAX_FILE_PATH_LEN);
    idx += MAX_FILE_PATH_LEN;
    // 播放倍速
    speed = pMsg->szBuff[idx++];

    //
    cam_idx = channel - 1;

    // 判断输入的INDEX 是否合法
    if (cam_idx < 0 || cam_idx >= MAX_CAMERA_COUNT)
    {
        cam_idx = 0;
    }

    // jt808 历史视频播放
    m_vctCam[cam_idx].startJt808FileVideo(phone_num, szIP, tcp_port, udp_port, channel, file_path);
}

// JT808历史推流控制
void CCameraManager::procJt808HisVideoControl(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    int idx = 0;
    BYTE channel = pMsg->szBuff[idx++];
    BYTE ctrl_code = pMsg->szBuff[idx++];
    BYTE stop_code = pMsg->szBuff[idx++];
    BYTE stream_code = pMsg->szBuff[idx++];

    int cam_idx = channel - 1;

    // 判断输入的INDEX 是否合法
    if (cam_idx < 0 || cam_idx >= MAX_CAMERA_COUNT)
    {
        cam_idx = 0;
    }

    switch (ctrl_code)
    {
    case 0: //停止推流
    {
        // 0和2状态表示关闭视频
        if (stop_code == 0 || stop_code == 2)
        {
            m_vctCam[cam_idx].stopJt808RTVideo();
        }
        break;
    }
    case 1: // 换码流
    {
        m_vctCam[cam_idx].changeJt808RTStreamCode(stream_code);
        break;
    }
    case 2: // 暂停推流
    {
        m_vctCam[cam_idx].pauseJt808RTVideo();
        break;
    }
    case 3: // 继续推流
    {
        m_vctCam[cam_idx].continueJt808RTVideo();
        break;
    }
    case 4:
    default:
        break;
    }
}

// 处理摄像头配置变更
void CCameraManager::procSetCamConfig(MODULE_MESSAGE *pMsg)
{
    int index = 0;
    CCamConfig cam_cfg;
    int offset = 0;
    memcpy(&index, pMsg->szBuff + offset, sizeof(int));
    offset += sizeof(int);

    memcpy(&cam_cfg, pMsg->szBuff + offset, sizeof(cam_cfg));
    offset += sizeof(cam_cfg);

    if (index >= 0 && index < MAX_CAMERA_COUNT)
    {
        m_vctCam[index].setCamConfig(cam_cfg);
    }
}

void CCameraManager::procSave2NewFile(MODULE_MESSAGE *pMsg)
{
    QDateTime tmBegin;
    QDateTime tmEnd;
    int interval = m_pCamDb->getValue("video_save_interval").toInt();
    memcpy(&tmBegin, pMsg->szBuff, sizeof(tmBegin));
    tmEnd = tmBegin.addSecs(interval);

    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        if (!m_vctCam[i].isCameraIdel())
        {
            m_vctCam[i].save2NewFile(tmBegin, tmEnd);
        }
    }
}

//实时摄像头任务，实时摄像头任务切换相关
void CCameraManager::CameraTask(void *pBuff)
{
    int i = 0;
    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        m_vctCam[i].startSnapshotTask();
    }
}

// 视频保存模式：按时间长度保存（1）; 按文件大小保存(2)
void CCameraManager::onSaveModeChanged(int mode)
{
    qDebug() << "onSaveModeChanged:" << mode;
}

// 数据位置
void CCameraManager::onDataPathChanged(QString path)
{
    qDebug() << "onDataPathChanged:" << path;
}

// 命名规则
void CCameraManager::onNameRuleChanged(QString rule)
{
    qDebug() << "onSaveInternalChanged:" << rule;
}

// 按时间长度保存的时间间隔(分钟)
void CCameraManager::onSaveInternalChanged(int val)
{
    qDebug() << "onSaveInternalChanged:" << val;
}

// 按文件大小保存的文件大小(MB)
void CCameraManager::onSaveSizeChanged(int val)
{
    qDebug() << "onSaveSizeChanged:" << val;
}

// 获取指定索引的设备地址
bool CCameraManager::onCamCfgChanged(int idx, CCamConfig &cam_cfg)
{
    qDebug() << "onCamCfgChanged:" << idx;
}

// 拆帧间隔
void CCameraManager::onFrameSplittingIntervalChanged(int frameInverval)
{
    qDebug() << "onFrameSplittingIntervalChanged:" << frameInverval;
}

// 查找局域网ip
void CCameraManager::tskCameraFind(void *pBuff)
{
    assert(m_pConfigMng);
    static bool bFinding = false;
	LOG_INFO(CCameraManager, "tskCameraFind START");

    if (!isShutdown() && bFinding == false)
    {
        m_mtxCamFind.lock();
        bFinding = true;
        m_webCamFinder.StopFind();
        m_webCamFinder.SetDiscoveryInfoCB(CCameraManager::updateCameraCallback, this);
        m_webCamFinder.StartFind();
        m_mtxCamFind.unlock();
    }
    bFinding = false;
}

// 获取空闲摄像头窗口
CCameraDevice *CCameraManager::getIdelCamera(int &index)
{
    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        if (m_vctCam[i].isCameraIdel())
        {
            index = i;
            return &m_vctCam[i];
        }
    }
    return nullptr;
}

// 获取指定源的摄像头窗口
CCameraDevice *CCameraManager::getCameraBySrc(QString strSrc)
{

    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        if (m_vctCam[i].getCamDev() == strSrc)
        {
            return &m_vctCam[i];
        }
    }
    return nullptr;
}

// 根据IP地址获取推荐的显示位
int CCameraManager::getPrefIdxByIP(QString strDev)
{
    int PrefIdx = 0;
    QStringList list = strDev.split('.');
    if (list.size() == 4)
    {
        PrefIdx = max(0, list[2].toInt() - 1);
    }
    return PrefIdx;
}
void CCameraManager::updateCamerapDevice(QString strDev)
{
    // 先判断指定IP是否在使用的设备列表中
    CCameraDevice *pCamera = getCameraBySrc(strDev);
    // 如果在，直接返回
    if (pCamera)
    {
        return;
    }
    // 如果不在,将摄像头放入一个指定的表
    else
    {
        QMutexLocker lock(&m_mtxCamera);
        CAM_CONFIG_MAP::Iterator it = m_mapCamConfig.find(strDev);
        CCamConfig cam_cfg(strDev);
        // 获取对应IP的配置信息
        if (it != m_mapCamConfig.end())
        {
            cam_cfg = *(it.value());
			
        }
        // 没有配置则初始化配置
        else
        {
            //
            cam_cfg.m_PrefIdx = getPrefIdxByIP(strDev);
            cam_cfg.setCamDev(strDev);
            setCamConfig(cam_cfg);
        }

        CCameraDevice *pDevice = nullptr;
        // 确认偏好索引的摄像头位置是否空闲
        int pref_idx = cam_cfg.m_PrefIdx;
        // 如果空闲，直接占用这个展示位
        if (pref_idx < MAX_CAMERA_COUNT && pref_idx >= 0 && m_vctCam[pref_idx].isCameraIdel())
        {
            pDevice = &m_vctCam[pref_idx];
            pDevice->m_PrefIdx = pref_idx;
        }
        // 如果不空闲，则获取一个空闲的摄像头位置
        else
        {
            pDevice = this->getIdelCamera(pref_idx);
            pDevice->m_PrefIdx = pref_idx;
        }

        // 开始运行
        if (pDevice)
        {
            pDevice->setCamConfig(cam_cfg);
            pDevice->setSource(cam_cfg.getVideoMain());
            pDevice->startRun();
			
            // TODO 广播摄像头上线事件
            MODULE_MESSAGE msg;
            int idx = 0;
            bool online = 1;
            msg.iCmd = CMD_CAM_ONLINE_OFFLINE;
            memcpy(msg.szBuff, &online, sizeof(bool));
            idx += sizeof(bool);
            memcpy(msg.szBuff + idx, &pDevice->m_PrefIdx, sizeof(int));
            idx += sizeof(int);

            m_pTcpModule->boardcastMessage(msg);
        }
    }
}

// 刷新设备列表
void CCameraManager::updateCameraCallback(DEVICE_IPC_INFO *pInfo, void *lpContext)
{
    CCameraManager *pThis = (CCameraManager *)lpContext;
    QString strDev = pInfo->IP;
    if (pThis != nullptr)
    {
        pThis->updateCamerapDevice(strDev);
    }
}

// 实时视频rtmp推送
bool CCameraManager::realtimeVideo(QVariantMap &prop_arr)
{
    bool bSuccess = false;
    // pushUrl	String	推流地址
    QString pushUrl = prop_arr["pushUrl"].toString();

    int cam_idx = max(0, prop_arr["camIdx"].toInt() - 1);

    if (cam_idx >= 0 && cam_idx < MAX_CAMERA_COUNT)
    {
        bSuccess = m_rtmpResponser.startRtmpRequire(m_vctCam[cam_idx].getVideoMain(), pushUrl);
    }

    onRealtimeVideo(pushUrl, bSuccess);

    // 开始推流处理
    return true;
}

// 实时视频推流的结果反馈
bool CCameraManager::onRealtimeVideo(QString pushUrl, bool bSuccess)
{
    assert(m_pWebSock);
    QVariantMap prop_arr;

    prop_arr.insert("msgName", "onRealtimeVideo");
    // pushUrl	String	推流地址
    prop_arr.insert("pushUrl", pushUrl);

    int retCode = bSuccess ? 1 : 0;
    // retCode	number	返回码，0表示成功
    prop_arr.insert("retCode", retCode);

    // message	String	返回消息
    prop_arr.insert("message", "");

    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }
    return true;
}

// 停止实时视频推流
bool CCameraManager::stopRealtimeVideo(QVariantMap &prop_arr)
{
    // pushUrl	String	推流地址
    QString pushUrl = prop_arr["pushUrl"].toString();

    m_rtmpResponser.stopRtmpRequire();
    // 停止推流处理
    return true;
}

// 抓拍
bool CCameraManager::snapShot(QVariantMap &prop_arr)
{
    // interval	number	多张抓拍之间的时间间距
    m_iSnapInterval = (int)prop_arr["interval"].toFloat();
    // number	number	抓拍的张数
    m_iSnapCount = (int)prop_arr["number"].toFloat();

    m_strSnapList.clear();

    if (m_iSnapCount > 0)
    {
        for (int i = 0; i < MAX_CAMERA_COUNT; i++)
        {
            if (!m_vctCam[i].isCameraIdel())
            {
                m_bSnapFinished[i] = false;
                m_vctCam[i].snapshot(m_iSnapInterval, m_iSnapCount);
            }
        }
    }
    else
    {
        onSnapShot();
    }

    return true;
}

bool CCameraManager::snapShotFinilished(int cam_idx, QString strFileList)
{
    bool bAllFinished = true;
    m_strSnapList += strFileList;

    if (cam_idx >= 0 && cam_idx < MAX_CAMERA_COUNT)
    {
        m_bSnapFinished[cam_idx] = true;
    }

    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        if (!m_vctCam[i].isCameraIdel() && m_bSnapFinished[i] == false)
        {
            bAllFinished = false;
            break;
        }
    }

    if (bAllFinished)
    {
        onSnapShot();
    }
}
// 抓拍的结果反馈
bool CCameraManager::onSnapShot()
{
    assert(m_pWebSock);
    QVariantMap prop_arr;

    prop_arr.insert("msgName", "onSnapShot");
    // interval	number	多张抓拍之间的时间间距
    prop_arr.insert("interval", QString("%1").arg(m_iSnapInterval));

    // number	number	抓拍的张数
    prop_arr.insert("number", QString("%1").arg(m_iSnapCount));

    // fileList	Array	文件列表；可以直接下载的url的列表
    prop_arr.insert("fileList", m_strSnapList);

    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }
    return true;
}
